#
# set 3 directories below, BSDSRCDIR, MODSRCDIR, INSTALLDIR.
#   BSDSRCDIR   : 4.4BSD (from CSRG CD-ROM #4) / Net2 source tree
#   MODSRCDIR   : modified tree, here this file is in.
#   INSTALLDIR  : compiled binaries will be installed to.
#
# 4.4BSD / Net2 self compile:
#   echo "ln -sf /usr/local/tools/bin/make /usr/local/bin" | su root
#   ## add /usr/local/bin to path for make, sudo and zsh.
#   make install-sudo
#   ## I recommend installing zsh.
#   (cd 40ex/zsh-2.5.03 && make depend && make && sudo -E make install
#     sudo -E ln -sf /usr/local/zsh-2.5.03/bin/zsh /usr/local/bin)
#
#   make install-bin
#   make install-kern
#
# semi-cross compile on NetBSD 9 with NetBSD 1.0's bin at /emul/aout
# 
#   *install sudo, e.g. pkg_add sudo
#
#   make install-hostbin
#
#   make install-bin
#   make install-kern

BSDSRCDIR ?=	/usr/src/4.4BSD-Alpha/usr/src
MODSRCDIR ?=	/path/to/b44a-extra-tree
INSTALLDIR ?=	/usr/obj

MAKESYSPATH ?=	${MODSRCDIR}/share/mk
DESTD0 ?=	${INSTALLDIR}/b44a-installed

# don't add -fschg to install
NOSCHG ?=	yes
.if defined(NOSCHG) && ${NOSCHG} != ""
NOSCHG_DEF =	NOSCHG=${NOSCHG}
.endif
SCHG_LIST =	/sbin/init /bin/rcp /usr/sbin/sliplogin /usr/bin/chpass \
		/usr/bin/login /usr/bin/passwd /usr/bin/rdist \
		/usr/bin/rlogin /usr/sbin/rsh /usr/sbin/su /usr/sbin/rkinit

SCHG_FILES =	${SCHG_LIST:S/^/${DESTDIR}/}

MKENV =	MAKESYSPATH=${MAKESYSPATH} BSDSRCDIR=${BSDSRCDIR} DESTD0=${DESTD0}

# SUBDIR1 =	lib contrib/flex-2.5.2 contrib/X11R5-lib local/kerberosIV
SUBDIR1 =	lib usr.bin/lex/lib kerberosIV usr.bin/gcc
SUBDIR2 =	bin sbin libexec usr.bin usr.sbin share old games

.if exists(sys/${MACHINE}/stand)
SUBDIR2 +=	sys/${MACHINE}/stand
.endif

####
.if ${MACHINE} == "i386"
SUBDIR2 +=	20nb/i386-boot-nb08
.endif

# SUBDIR2 +=	10mod 30tools
SUBDIR2 +=	10mod

STAMPS =	${SUBDIR1:S/\//-/g:S/^/stamp-/g}

# HOSTBIN_SUBDIRS =	10mod/pwd_mkdb \
# 			usr.bin/cap_mkdb usr.sbin/mtree usr.bin/lorder
HOSTBIN_SUBDIRS =	10mod/pwd_mkdb usr.sbin/mtree usr.bin/lorder
HOSTBIN_SUBDIRS +=	usr.bin/lex

.if ${MACHINE} == "sparc"
HOSTBIN_SUBDIRS +=	10mod/config.new usr.bin/ld
HOSTBIN_DEPEND =	stamp-include
.elif ${MACHINE} == "mips" || ${MACHINE} == "news3400"
HOSTBIN_SUBDIRS +=	10mod/config
HOSTBIN_SUBDIRS +=	30tools
.elif ${MACHINE} == "luna68k" || ${MACHINE} == "hp300" || ${MACHINE} == "i386"
HOSTBIN_SUBDIRS +=	10mod/config usr.bin/ld
# HOSTBIN_DEPEND =	stamp-include
.endif

.if exists(/usr/local/bin/sudo) || exists(/usr/pkg/bin/sudo)
SU_CMD ?=	sudo -E
.endif

install-kern: cleandir-kern
	cd 80sys/kern && ${MKENV} make depend
	cd 80sys/kern && ${MKENV} make
	cd 80sys/kern && ${MKENV} ${SU_CMD} make install

cleandir-kern:
	cd 80sys/kern && ${MKENV} make cleandir

install-bin:  stamp-etc stamp-include ${SUBDIR1:S/\//-/g:S/^/stamp-/g}
	for d in ${SUBDIR2}; do \
	  echo '----> '$${d}; \
	  (cd $${d} && \
	    ${MKENV} make depend; \
	    ${MKENV} make; \
	    ${MKENV} ${SU_CMD} make ${NOSCHG_DEF} install); \
	done

##----------------------
# .for SUBD in ${SUBDIR1}
# ${SUBD:S/\//-/g:S/^/stamp-/g}:
# 	echo '----> '${SUBD}
# 	cd ${SUBD} && ${MKENV} make depend
# 	cd ${SUBD} && ${MKENV} make
# 	cd ${SUBD} && ${MKENV} ${SU_CMD} make ${NOSCHG_DEF} install
# 	touch ${.TARGET}
# .endfor

SUBD1_CMD =	subd=${.TARGET:S/^stamp-//:S/-/\//g}; echo '----> '$${subd}; \
		(cd $${subd} && ${MKENV} make depend && \
		  ${MKENV} make && \
		  ${MKENV} ${SU_CMD} make ${NOSCHG_DEF} install) && \
		touch ${.TARGET}

stamp-lib:
	${SUBD1_CMD}
stamp-usr.bin-lex-lib:
	${SUBD1_CMD}
stamp-kerberosIV:
	${SUBD1_CMD}
stamp-usr.bin-gcc:
	${SUBD1_CMD}

##----------------------

install-libc:
	for d in lib/libc lib/csu/${MACHINE}; do \
	  echo '----> '$${d}; \
	  (cd $${d} && \
	    ${MKENV} make cleandir; \
	    ${MKENV} make NOMAN=1 NOPROFILE=1 depend; \
	    ${MKENV} make NOMAN=1 NOPROFILE=1; \
	    ${MKENV} ${SU_CMD} make NOMAN=1 NOPROFILE=1 install); \
	done

set-schg:
	${SU_CMD} chflags schg ${SCHG_FILES}

cleandir-bin:
	rm -rf stamp-etc stamp-include ${STAMPS}
	for d in ${SUBDIR1} ${SUBDIR2}; do \
	  echo '----> '$${d}; \
	  (cd $${d} && ${MKENV} make cleandir); \
	done

stamp-etc:
	cd etc && ${MKENV} ${SU_CMD} make distribution
	touch ${.TARGET}

install-etc:
	cd etc && ${MKENV} ${SU_CMD} make distribution

stamp-include:
	cd include && ${MKENV} ${SU_CMD} make install
	touch ${.TARGET}

install-include:
	cd include && ${MKENV} ${SU_CMD} make install

#- hostbin
install-hostbin:  ${HOSTBIN_DEPEND}
	for d in ${HOSTBIN_SUBDIRS}; do \
	  echo '----> '$${d}; \
	  (cd $${d} && \
	    ${MKENV} make HOST_BIN=yes depend; \
	    ${MKENV} make HOST_BIN=yes; \
	    ${MKENV} ${SU_CMD} make ${NOSCHG_DEF} HOST_BIN=yes install; \
	    ${MKENV} make HOST_BIN=yes cleandir); \
	done

cleandir-hostbin:
	for d in ${HOSTBIN_SUBDIRS}; do \
	  echo '----> '$${d}; \
	  (cd $${d} && ${MKENV} make HOST_BIN=yes cleandir); \
	done

#- make
install-make:
	d=10mod/make; \
	(cd $${d} && \
	  ${MKENV} make HOST_BIN=yes depend; \
	  ${MKENV} make HOST_BIN=yes; \
	  ${MKENV} ${SU_CMD} make ${NOSCHG_DEF} HOST_BIN=yes install; \
	  ${SU_CMD} ln -sf \
		/usr/local/cross/${MACHINE}-bsd44f/bin/make /usr/local/bin; \
	  ${MKENV} make HOST_BIN=yes cleandir)

cleandir-make:
	cd 10mod/make && ${MKENV} make HOST_BIN=yes cleandir

#- sudo
SUDO_SRC =	40ex/sudo-1.6.9p23
SUDO_BIN =	${SUDO_SRC}/_workdir/sudo

sudo: ${SUDO_BIN}

${SUDO_BIN}:
	cd ${SUDO_SRC} && make cleandir && make depend && make

install-sudo: ${SUDO_BIN}
.if !defined(SU_CMD)
	cd ${SUDO_SRC} && echo "make install" | su root
.else
	cd ${SUDO_SRC} && ${SU_CMD} make install
.endif

cleandir-sudo:
	cd ${SUDO_SRC} && make cleandir

